@@ -30,6 +30,10 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#ifdef GDK_WINDOWING_QUARTZ
+#include <ige-mac-integration.h>
+#endif
+
 #include <libgossip/gossip-chatroom.h>
 #include <libgossip/gossip-contact.h>
 #include <libgossip/gossip-event.h>
@@ -39,8 +43,9 @@
 #include <libgossip/gossip-conf.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-stock.h>
+#include <libgossip/gossip-jabber.h>
+#include <libgossip/gossip-jabber-utils.h>
 
-#include "gossip-about-dialog.h"
 #include "gossip-accounts-dialog.h"
 #include "gossip-add-contact-dialog.h"
 #include "gossip-app.h"
@@ -68,6 +73,10 @@
 #include "gossip-vcard-dialog.h"
 #include "ephy-spinner.h"
 
+#ifdef GDK_WINDOWING_QUARTZ
+#include "gossip-mac-dock.h"
+#endif
+
 #ifdef HAVE_DBUS
 #include "gossip-dbus.h"
 #endif
@@ -113,7 +122,8 @@ struct _GossipAppPriv {
 	/* Main widgets */
 	GtkWidget             *window;
 	GtkWidget             *main_vbox;
-	GtkWidget             *filter_entry;
+	GtkWidget             *find_hbox;
+	GtkWidget             *find_entry;
 	GtkWidget             *errors_vbox;
 	GHashTable            *errors;
 	GHashTable            *reconnects;
@@ -121,16 +131,18 @@ struct _GossipAppPriv {
 	/* Tooltips for all widgets */
 	GtkTooltips           *tooltips;
 
+	GtkWidget             *menu_bar;
+
 	/* Menu widgets */
 	GtkWidget             *chat_connect;
 	GtkWidget             *chat_disconnect;
 	GtkWidget             *chat_search;
+	GtkWidget             *chat_context;
+	GtkWidget             *chat_context_separator;
 	GtkWidget             *room;
 	GtkWidget             *room_menu;
 	GtkWidget             *room_sep;
 	GtkWidget             *room_join_favorites;
-	GtkWidget             *edit_context;
-	GtkWidget             *edit_context_separator;
 
 	/* Status Icon */
 	GtkStatusIcon         *status_icon;
@@ -139,6 +151,11 @@ struct _GossipAppPriv {
 	GtkWidget             *popup_menu_status_item;
 	GtkWidget             *popup_menu_show_list_item;
 
+	/* Dock */
+#ifdef GDK_WINDOWING_QUARTZ
+	GossipDock            *dock;
+#endif
+
 	/* Throbber */
 	GtkWidget             *throbber;
 
@@ -175,7 +192,11 @@ static gboolean
 app_main_window_key_press_event_cb           (GtkWidget             *window,
 					      GdkEventKey           *event,
 					      GossipApp             *app);
-static void     app_filter_entry_changed_cb  (GtkEditable           *editable,
+static gboolean
+app_main_window_focus_in_event_cb            (GtkWidget             *widget,
+					      GdkEvent              *event,
+					      GossipApp             *app);
+static void     app_find_entry_changed_cb    (GtkEditable           *editable,
 					      GossipApp             *app);
 static void     
 app_favorite_chatroom_menu_activate_cb       (GtkMenuItem           *menu_item,
@@ -188,6 +209,10 @@ static void
 app_favorite_chatroom_menu_update_cb         (GossipChatroomManager *manager,
 					      GossipChatroom        *chatroom,
 					      gpointer               user_data);
+static void
+app_favorite_chatroom_menu_name_cb           (GossipChatroom        *chatroom,
+					      GParamSpec            *spec,
+					      GtkWidget             *menu_item);
 static void     
 app_favorite_chatroom_menu_added_cb          (GossipChatroomManager *manager,
 					      GossipChatroom        *chatroom,
@@ -200,10 +225,6 @@ static void
 app_favorite_chatroom_menu_setup             (void);
 static void     app_chat_quit_cb             (GtkWidget             *window,
 					      GossipApp             *app);
-static void     app_chat_connect_cb          (GtkWidget             *window,
-					      GossipApp             *app);
-static void     app_chat_disconnect_cb       (GtkWidget             *window,
-					      GossipApp             *app);
 static void     app_chat_search_cb           (GtkWidget             *window,
 					      GossipApp             *app);
 static void     app_chat_new_message_cb      (GtkWidget             *widget,
@@ -221,7 +242,7 @@ static void     app_chat_add_contact_cb      (GtkWidget             *widget,
 static void     app_chat_show_offline_cb     (GtkCheckMenuItem      *item,
 					      GossipApp             *app);
 static gboolean 
-app_edit_button_press_event_cb               (GtkWidget             *widget,
+app_chat_button_press_event_cb               (GtkWidget             *widget,
 					      GdkEventButton        *event,
 					      GossipApp             *app);
 static void     app_edit_accounts_cb         (GtkWidget             *widget,
@@ -263,9 +284,7 @@ app_session_protocol_error_cb                (GossipSession         *session,
 					      GossipAccount         *account,
 					      GError                *error,
 					      gpointer               user_data);
-static gchar *  app_session_get_password_cb  (GossipSession         *session,
-					      GossipAccount         *account,
-					      gpointer               user_data);
+static void app_account_password_dialog_show (GossipAccount         *account);
 static void     app_restore_user_accels      (void);
 static void     app_save_user_accels         (void);
 static void     app_show_hide_list_cb        (GtkWidget             *widget,
@@ -277,8 +296,10 @@ app_status_icon_popup_menu_cb                (GtkStatusIcon         *status_icon
 					      guint                  button,
 					      guint                  activate_time,
 					      GossipApp             *app);
-static void     app_status_icon_create_menu  (void);
+static GtkWidget *
+                app_status_icon_create_menu  (GossipApp             *app);
 static void     app_status_icon_create       (void);
+static void     app_setup_dock               (void);
 static gboolean 
 app_status_icon_check_embedded_cb            (gpointer               user_data);
 static void     app_notify_show_offline_cb   (GossipConf            *conf,
@@ -294,7 +315,6 @@ static void
 app_notify_compact_contact_list_cb           (GossipConf            *conf,
 					      const gchar           *key,
 					      gpointer               user_data);
-static void     app_disconnect               (void);
 static void     app_connection_items_setup   (GladeXML              *glade);
 static void     app_connection_items_update  (void);
 static void     
@@ -326,6 +346,9 @@ app_session_chatroom_auto_connect_cb         (GossipSession         *session,
 static void     app_event_added_cb           (GossipEventManager    *manager,
 					      GossipEvent           *event,
 					      gpointer               user_data);
+static void     app_accounts_updated_cb      (GossipAccountManager  *manager,
+					      GossipAccount         *account,
+					      gpointer               user_data);
 static void     app_contact_activated_cb     (GossipContactList     *contact_list,
 					      GossipContact         *contact,
 					      GossipEventId          event_id,
@@ -408,7 +431,9 @@ app_finalize (GObject *object)
 		g_hash_table_destroy (priv->reconnects);
 	}
 
-	gtk_widget_destroy (priv->popup_menu);
+	if (priv->popup_menu) {
+		gtk_widget_destroy (priv->popup_menu);
+	}
 
 	g_object_unref (priv->tooltips);
 
@@ -432,9 +457,6 @@ app_finalize (GObject *object)
 	g_signal_handlers_disconnect_by_func (priv->session,
 					      app_session_protocol_error_cb,
 					      NULL);
-	g_signal_handlers_disconnect_by_func (priv->session,
-					      app_session_get_password_cb,
-					      NULL);
 	g_signal_handlers_disconnect_by_func (priv->session,
 					      app_session_chatroom_auto_connect_cb,
 					      NULL);
@@ -443,6 +465,10 @@ app_finalize (GObject *object)
 					      app_event_added_cb,
 					      NULL);
 
+	g_signal_handlers_disconnect_by_func (gossip_session_get_account_manager (priv->session),
+					      app_accounts_updated_cb,
+					      NULL);
+
 	gossip_ft_dialog_finalize (priv->session);
 	gossip_subscription_dialog_finalize (priv->session);
 	
@@ -490,7 +516,7 @@ app_setup_throbber (void)
 	GossipAppPriv *priv;
 	GtkWidget     *ebox;
 	GtkToolItem   *item;
-	gchar         *str;
+	const gchar   *str;
 
 	priv = GET_PRIV (app);
 
@@ -691,12 +717,14 @@ app_setup_contact_list (GtkWidget *scrolled_window,
 static void
 app_setup (GossipSession *session)
 {
-	GossipAppPriv *priv;
-	GossipConf    *conf;
-	GladeXML      *glade;
-	GtkWidget     *sw;
-	GtkWidget     *show_offline_widget;
-
+	GossipAppPriv        *priv;
+	GossipConf           *conf;
+	GossipAccountManager *account_manager;
+	GladeXML             *glade;
+	GtkWidget            *sw;
+	GtkWidget            *show_offline_widget;
+	GtkWidget            *help_contents;
+			
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Beginning...");
 
 	priv = GET_PRIV (app);
@@ -705,6 +733,8 @@ app_setup (GossipSession *session)
 
 	priv->session = g_object_ref (session);
 
+	account_manager = gossip_session_get_account_manager (session);
+
 	gossip_debug (DEBUG_DOMAIN_SETUP,
 		      "Initialising session listeners "
 		      "(subscription, file transfer, etc)");
@@ -741,23 +771,15 @@ app_setup (GossipSession *session)
 	g_signal_connect (priv->session, "protocol-connecting",
 			  G_CALLBACK (app_session_protocol_connecting_cb),
 			  NULL);
-
 	g_signal_connect (priv->session, "protocol-connected",
 			  G_CALLBACK (app_session_protocol_connected_cb),
 			  NULL);
-
 	g_signal_connect (priv->session, "protocol-disconnected",
 			  G_CALLBACK (app_session_protocol_disconnected_cb),
 			  NULL);
-
 	g_signal_connect (priv->session, "protocol-error",
 			  G_CALLBACK (app_session_protocol_error_cb),
 			  NULL);
-
-	g_signal_connect (priv->session, "get-password",
-			  G_CALLBACK (app_session_get_password_cb),
-			  NULL);
-
 	g_signal_connect (priv->session, "chatroom-auto-connect",
 			  G_CALLBACK (app_session_chatroom_auto_connect_cb),
 			  NULL);
@@ -766,6 +788,13 @@ app_setup (GossipSession *session)
 			  G_CALLBACK (app_event_added_cb),
 			  NULL);
 
+	g_signal_connect (account_manager, "account-added",
+			  G_CALLBACK (app_accounts_updated_cb),
+			  NULL);
+	g_signal_connect (account_manager, "account-removed",
+			  G_CALLBACK (app_accounts_updated_cb),
+			  NULL);
+
 	/* Set up interface */
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Initialising interface");
 	glade = gossip_glade_get_file ("main.glade",
@@ -773,17 +802,20 @@ app_setup (GossipSession *session)
 				       NULL,
 				       "main_window", &priv->window,
 				       "main_vbox", &priv->main_vbox,
-				       "filter_entry", &priv->filter_entry,
+				       "main_menubar", &priv->menu_bar,
+				       "find_hbox", &priv->find_hbox,
+				       "find_entry", &priv->find_entry,
 				       "errors_vbox", &priv->errors_vbox,
 				       "chat_connect", &priv->chat_connect,
 				       "chat_disconnect", &priv->chat_disconnect,
 				       "chat_search", &priv->chat_search,
+				       "chat_context", &priv->chat_context,
+				       "chat_context_separator", &priv->chat_context_separator,
 				       "chat_show_offline", &show_offline_widget,
 				       "room", &priv->room,
 				       "room_sep", &priv->room_sep,
 				       "room_join_favorites", &priv->room_join_favorites,
-				       "edit_context", &priv->edit_context,
-				       "edit_context_separator", &priv->edit_context_separator,
+				       "help_contents", &help_contents,
 				       "presence_toolbar", &priv->presence_toolbar,
 				       "roster_scrolledwindow", &sw,
 				       NULL);
@@ -794,10 +826,10 @@ app_setup (GossipSession *session)
 			      "main_window", "delete_event", app_main_window_delete_event_cb,
 			      "main_window", "configure_event", app_window_configure_event_cb,
 			      "main_window", "key_press_event", app_main_window_key_press_event_cb,
-			      "filter_entry", "changed", app_filter_entry_changed_cb, 
+			      "main_window", "focus-in-event", app_main_window_focus_in_event_cb,
+			      "find_entry", "changed", app_find_entry_changed_cb, 
+			      "chat", "button-press-event", app_chat_button_press_event_cb,
 			      "chat_quit", "activate", app_chat_quit_cb,
-			      "chat_connect", "activate", app_chat_connect_cb,
-			      "chat_disconnect", "activate", app_chat_disconnect_cb,
 			      "chat_search", "activate", app_chat_search_cb,
 			      "chat_new_message", "activate", app_chat_new_message_cb,
 			      "chat_history", "activate", app_chat_history_cb,
@@ -806,7 +838,6 @@ app_setup (GossipSession *session)
 			      "room_manage_favorites", "activate", app_room_manage_favorites_cb,
 			      "chat_add_contact", "activate", app_chat_add_contact_cb,
 			      "chat_show_offline", "toggled", app_chat_show_offline_cb,
-			      "edit", "button-press-event", app_edit_button_press_event_cb,
 			      "edit_accounts", "activate", app_edit_accounts_cb,
 			      "edit_personal_information", "activate", app_edit_personal_information_cb,
 			      "edit_preferences", "activate", app_edit_preferences_cb,
@@ -817,8 +848,48 @@ app_setup (GossipSession *session)
 	/* Set up menu */
 	app_favorite_chatroom_menu_setup ();
 
-	gtk_widget_hide (priv->edit_context);
-	gtk_widget_hide (priv->edit_context_separator);
+#ifdef GDK_WINDOWING_QUARTZ
+	{
+		GtkWidget       *item;
+		IgeMacMenuGroup *group;
+
+		gtk_widget_set_name (priv->window, "small-font");
+
+                /*ige_mac_menu_connect_window_key_handler (GTK_WINDOW (priv->window));*/
+		ige_mac_menu_set_global_key_handler_enabled (TRUE);
+
+		ige_mac_menu_set_menu_bar (GTK_MENU_SHELL (priv->menu_bar));
+
+		item = glade_xml_get_widget (glade, "chat_quit");
+		ige_mac_menu_set_quit_menu_item (GTK_MENU_ITEM (item));
+
+		group =  ige_mac_menu_add_app_menu_group ();
+		item = glade_xml_get_widget (glade, "help_about");
+		ige_mac_menu_add_app_menu_item (group, GTK_MENU_ITEM (item),
+						_("About Gossip"));
+
+		group =  ige_mac_menu_add_app_menu_group ();
+		item = glade_xml_get_widget (glade, "edit_preferences");
+		ige_mac_menu_add_app_menu_item (group, GTK_MENU_ITEM (item),
+						_("Preferences..."));
+
+		/* Hide the left over separators. */
+		item = glade_xml_get_widget (glade, "quit_separator");
+		gtk_widget_hide (item);
+
+		item = glade_xml_get_widget (glade, "preferences_separator");
+		gtk_widget_hide (item);
+
+		gtk_widget_hide (priv->menu_bar);
+	}
+#endif
+
+	gtk_widget_hide (priv->chat_context);
+	gtk_widget_hide (priv->chat_context_separator);
+
+#ifndef HAVE_HELP
+	gtk_widget_hide (help_contents);
+#endif
 
 	/* Set up connection related widgets. */
 	app_connection_items_setup (glade);
@@ -831,10 +902,8 @@ app_setup (GossipSession *session)
 	
 	app_setup_throbber ();
 
-
 	/* Set up notification area / tray. */
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Configuring notification area widgets");
-	app_status_icon_create_menu ();
 	app_status_icon_create ();
 
 	app_setup_presences ();
@@ -846,7 +915,8 @@ app_setup (GossipSession *session)
 	/* Setup the contact list */
 	app_setup_contact_list (sw, show_offline_widget);
 
-	/* Sort criterium */
+	app_setup_dock ();
+
 	/* Set window to be hidden. If doesn't have status icon, show window
 	 * and mask "chat_hide_list".
 	 */
@@ -1058,6 +1128,26 @@ app_main_window_key_press_event_cb (GtkWidget   *window,
 				    GdkEventKey *event,
 				    GossipApp   *app)
 {
+	GossipAppPriv *priv;
+
+	priv = GET_PRIV (app);
+
+	/* Only do the following actions if escape is pressed */
+	if (event->keyval != GDK_Escape) {
+		return FALSE;
+	}
+
+	/* First if we have the find entry up, clear the filter and
+	 * hide the widgets.
+	 */
+	if (GTK_WIDGET_VISIBLE (priv->find_hbox)) {
+		gtk_entry_set_text (GTK_ENTRY (priv->find_entry), "");
+		gtk_widget_hide (priv->find_hbox);
+
+		return FALSE;
+	}
+
+	/* Second assume we want to hide the window */
 	if (event->keyval == GDK_Escape) {
 		gossip_app_toggle_visibility ();
 	}
@@ -1065,16 +1155,32 @@ app_main_window_key_press_event_cb (GtkWidget   *window,
 	return FALSE;
 }
 
+static gboolean
+app_main_window_focus_in_event_cb (GtkWidget *window,
+				   GdkEvent  *event,
+				   GossipApp *app)
+{
+	GossipAppPriv *priv;
+
+	priv = GET_PRIV (app);
+
+#ifdef GDK_WINDOWING_QUARTZ
+	ige_mac_menu_set_menu_bar (GTK_MENU_SHELL (priv->menu_bar));
+#endif
+
+	return FALSE;
+}
+
 static void
-app_filter_entry_changed_cb (GtkEditable *editable,
-			     GossipApp   *app)
+app_find_entry_changed_cb (GtkEditable *editable,
+		     	   GossipApp   *app)
 {
 	GossipAppPriv *priv;
 	const gchar   *filter;
 
 	priv = GET_PRIV (app);
 
-	filter = gtk_entry_get_text (GTK_ENTRY (priv->filter_entry));
+	filter = gtk_entry_get_text (GTK_ENTRY (priv->find_entry));
 	gossip_contact_list_set_filter (priv->contact_list, filter);
 }
 
@@ -1114,7 +1220,7 @@ app_favorite_chatroom_menu_update (void)
 		account = gossip_chatroom_get_account (chatroom);
 		menu_item = g_object_get_data (G_OBJECT (chatroom), "menu_item");
 
-		visible = gossip_chatroom_get_favourite (chatroom);
+		visible = gossip_chatroom_get_favorite (chatroom);
 		visible &= gossip_session_is_connected (priv->session, account);
 
 		if (visible) {
@@ -1150,7 +1256,7 @@ app_favorite_chatroom_menu_add (GossipChatroom *chatroom)
 	account = gossip_chatroom_get_account (chatroom);
 	menu_item = g_object_get_data (G_OBJECT (chatroom), "menu_item");
 
-	visible = gossip_chatroom_get_favourite (chatroom);
+	visible = gossip_chatroom_get_favorite (chatroom);
 	visible &= gossip_session_is_connected (priv->session, account);
 
 	if (menu_item) {
@@ -1164,6 +1270,9 @@ app_favorite_chatroom_menu_add (GossipChatroom *chatroom)
 	g_signal_connect (menu_item, "activate",
 			  G_CALLBACK (app_favorite_chatroom_menu_activate_cb),
 			  chatroom);
+	g_signal_connect (chatroom, "notify::name",
+			  G_CALLBACK (app_favorite_chatroom_menu_name_cb),
+			  menu_item);
 
 	gtk_menu_shell_insert (GTK_MENU_SHELL (priv->room_menu),
 			       menu_item, 3);
@@ -1183,6 +1292,17 @@ app_favorite_chatroom_menu_update_cb (GossipChatroomManager *manager,
 	app_favorite_chatroom_menu_update ();
 }
 
+static void
+app_favorite_chatroom_menu_name_cb (GossipChatroom *chatroom,
+				    GParamSpec     *spec,
+				    GtkWidget      *menu_item)
+{
+	GtkWidget *label;
+
+	label = gtk_bin_get_child (GTK_BIN (menu_item));
+	gtk_label_set_text (GTK_LABEL (label), gossip_chatroom_get_name (chatroom));
+}
+
 static void
 app_favorite_chatroom_menu_added_cb (GossipChatroomManager *manager,
 				     GossipChatroom        *chatroom,
@@ -1213,9 +1333,13 @@ app_favorite_chatroom_menu_removed_cb (GossipChatroomManager *manager,
 	account = gossip_chatroom_get_account (chatroom);
 	menu_item = g_object_get_data (G_OBJECT (chatroom), "menu_item");
 
-	visible = gossip_chatroom_get_favourite (chatroom);
+	visible = gossip_chatroom_get_favorite (chatroom);
 	visible &= gossip_session_is_connected (priv->session, account);
 
+	g_signal_handlers_disconnect_by_func (chatroom, 
+					      G_CALLBACK (app_favorite_chatroom_menu_name_cb),
+					      menu_item);
+
 	g_object_set_data (G_OBJECT (chatroom), "menu_item", NULL);
 	gtk_widget_destroy (menu_item);
 
@@ -1237,6 +1361,13 @@ app_favorite_chatroom_menu_setup (void)
 	priv->room_menu =
 		gtk_menu_item_get_submenu (GTK_MENU_ITEM (priv->room));
 
+	/* Reverse the list here because we want to make sure that we
+	 * always insert the next item in at position 3 so we start at
+	 * the right place. This of course means that new items are
+	 * put in before the last one.
+	 */
+	chatrooms = g_list_reverse (chatrooms);
+
 	for (l = chatrooms; l; l = l->next) {
 		if (app_favorite_chatroom_menu_add (l->data)) {
 			found = TRUE;
@@ -1251,7 +1382,7 @@ app_favorite_chatroom_menu_setup (void)
 
 	gtk_widget_set_sensitive (priv->room_join_favorites, found);
 
-	g_signal_connect (priv->chatroom_manager, "chatroom-favourite-update",
+	g_signal_connect (priv->chatroom_manager, "chatroom-favorite-update",
 			  G_CALLBACK (app_favorite_chatroom_menu_update_cb),
 			  NULL);
 	g_signal_connect (priv->chatroom_manager, "chatroom-added",
@@ -1262,6 +1393,26 @@ app_favorite_chatroom_menu_setup (void)
 			  NULL);
 }
 
+static void
+app_chat_connect_activate_cb (GtkMenuItem *menuitem,
+			      gpointer     user_data)
+{
+	GossipAccount *account;
+
+	account = g_object_get_data (G_OBJECT (menuitem), "account");
+	gossip_app_connect (account, FALSE); 
+}
+
+static void
+app_chat_disconnect_activate_cb (GtkMenuItem *menuitem,
+				 gpointer     user_data)
+{
+	GossipAccount *account;
+
+	account = g_object_get_data (G_OBJECT (menuitem), "account");
+	gossip_app_disconnect (account); 
+}
+
 static void
 app_chat_quit_cb (GtkWidget *window,
 		  GossipApp *app)
@@ -1275,20 +1426,6 @@ app_chat_quit_cb (GtkWidget *window,
 	}
 }
 
-static void
-app_chat_connect_cb (GtkWidget *window,
-		     GossipApp *app)
-{
-	gossip_app_connect (NULL, FALSE);
-}
-
-static void
-app_chat_disconnect_cb (GtkWidget *window,
-			GossipApp *app)
-{
-	app_disconnect ();
-}
-
 static void
 app_chat_search_cb (GtkWidget *window,
 		    GossipApp *app)
@@ -1297,11 +1434,11 @@ app_chat_search_cb (GtkWidget *window,
 
 	priv = GET_PRIV (app);
 	
-	if (GTK_WIDGET_VISIBLE (priv->filter_entry)) {
-		gtk_widget_hide (priv->filter_entry);
+	if (GTK_WIDGET_VISIBLE (priv->find_hbox)) {
+		gtk_widget_hide (priv->find_hbox);
 	} else {
-		gtk_widget_show (priv->filter_entry);
-		gtk_widget_grab_focus (priv->filter_entry);
+		gtk_widget_show (priv->find_hbox);
+		gtk_widget_grab_focus (priv->find_entry);
 	}
 }
 
@@ -1353,7 +1490,11 @@ static void
 app_room_manage_favorites_cb (GtkWidget *window,
 			      GossipApp *app)
 {
-	gossip_chatrooms_window_show (NULL, FALSE);
+	GossipAppPriv *priv;
+
+	priv = GET_PRIV (app);
+
+	gossip_chatrooms_window_show (GTK_WINDOW (priv->window), FALSE);
 }
 
 static void
@@ -1367,6 +1508,29 @@ app_chat_add_contact_cb (GtkWidget *widget,
 	gossip_add_contact_dialog_show (GTK_WINDOW (priv->window), NULL);
 }
 
+static void
+app_setup_dock (void)
+{
+#ifdef GDK_WINDOWING_QUARTZ
+	GossipAppPriv *priv;
+	GossipConf    *conf;
+	gboolean       hidden;
+
+	priv = GET_PRIV (app);
+
+	priv->dock = gossip_dock_get ();
+
+	conf = gossip_conf_get ();
+	gossip_conf_get_bool (conf,
+			      GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN,
+			      &hidden);
+
+	if (!hidden) {
+		gtk_widget_show (priv->window);
+	}
+#endif
+}
+
 static void
 app_chat_show_offline_cb (GtkCheckMenuItem *item,
 			  GossipApp        *app)
@@ -1386,7 +1550,7 @@ app_chat_show_offline_cb (GtkCheckMenuItem *item,
 }
 
 static gboolean
-app_edit_button_press_event_cb (GtkWidget      *widget,
+app_chat_button_press_event_cb (GtkWidget      *widget,
 				GdkEventButton *event,
 				GossipApp      *app)
 {
@@ -1406,12 +1570,12 @@ app_edit_button_press_event_cb (GtkWidget      *widget,
 		GtkWidget   *label;
 		GtkWidget   *submenu;
 
-		item = GTK_MENU_ITEM (priv->edit_context);
+		item = GTK_MENU_ITEM (priv->chat_context);
 		label = gtk_bin_get_child (GTK_BIN (item));
 		gtk_label_set_text (GTK_LABEL (label), _("Group"));
 
-		gtk_widget_show (priv->edit_context);
-		gtk_widget_show (priv->edit_context_separator);
+		gtk_widget_show (priv->chat_context);
+		gtk_widget_show (priv->chat_context_separator);
 
 		submenu = gossip_contact_list_get_group_menu (priv->contact_list);
 		gtk_menu_item_set_submenu (item, submenu);
@@ -1427,12 +1591,12 @@ app_edit_button_press_event_cb (GtkWidget      *widget,
 		GtkWidget   *label;
 		GtkWidget   *submenu;
 
-		item = GTK_MENU_ITEM (priv->edit_context);
+		item = GTK_MENU_ITEM (priv->chat_context);
 		label = gtk_bin_get_child (GTK_BIN (item));
 		gtk_label_set_text (GTK_LABEL (label), _("Contact"));
 
-		gtk_widget_show (priv->edit_context);
-		gtk_widget_show (priv->edit_context_separator);
+		gtk_widget_show (priv->chat_context);
+		gtk_widget_show (priv->chat_context_separator);
 
 		submenu = gossip_contact_list_get_contact_menu (priv->contact_list,
 								contact);
@@ -1443,8 +1607,8 @@ app_edit_button_press_event_cb (GtkWidget      *widget,
 		return FALSE;
 	}
 
-	gtk_widget_hide (priv->edit_context);
-	gtk_widget_hide (priv->edit_context_separator);
+	gtk_widget_hide (priv->chat_context);
+	gtk_widget_hide (priv->chat_context_separator);
 
 	return FALSE;
 }
@@ -1479,10 +1643,35 @@ app_help_about_cb (GtkWidget *window,
 		   GossipApp *app)
 {
 	GossipAppPriv *priv;
+	GladeXML      *glade;
+	GtkWidget     *about_dialog;
 
 	priv = GET_PRIV (app);
 
-	gossip_about_dialog_new (GTK_WINDOW (priv->window));
+	glade = gossip_glade_get_file ("main.glade",
+				       "about_dialog",
+				       NULL,
+				       "about_dialog", &about_dialog,
+				       NULL);
+
+	g_object_unref (glade);
+
+	if (!about_dialog) {
+		g_warning ("Could not find about dialog in main glade file");
+		return;
+	}
+
+
+	gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (about_dialog),
+				      PACKAGE_VERSION);
+	gtk_about_dialog_set_logo_icon_name (GTK_ABOUT_DIALOG (about_dialog),
+					     PACKAGE_TARNAME);
+
+	g_signal_connect_swapped (about_dialog, "response", 
+				  G_CALLBACK (gtk_widget_destroy),
+				  about_dialog);
+
+	gtk_widget_show (about_dialog);
 }
 
 static void
@@ -1573,10 +1762,10 @@ app_session_protocol_connected_cb (GossipSession *session,
 static void
 app_reconnect_remove (gpointer data)
 {
-	guint *id;
+	guint id;
 
-	id = (guint*) data;
-	g_source_remove (*id);
+	id = GPOINTER_TO_UINT (data);
+	g_source_remove (id);
 }
 
 static gboolean
@@ -1647,7 +1836,7 @@ app_session_protocol_disconnected_cb (GossipSession *session,
 				    account);
 		g_hash_table_insert (priv->reconnects,
 				     g_object_ref (account),
-				     &id);
+				     GUINT_TO_POINTER (id));
 	}
 }
 
@@ -1676,23 +1865,116 @@ app_session_protocol_error_cb (GossipSession  *session,
 		ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
 	}
 
+	if (error && error->code == GOSSIP_JABBER_NO_PASSWORD) {
+		/* Handle this less generally. */
+		app_account_password_dialog_show (account);
+		return;
+	}
+
 	app_accounts_error_display (account, error);
 }
 
-static gchar *
-app_session_get_password_cb (GossipSession *session,
-			     GossipAccount *account,
-			     gpointer       user_data)
+static void
+app_account_password_dialog_activate_cb (GtkWidget *entry, 
+					 GtkDialog *dialog)
 {
-	GossipAppPriv *priv;
-	gchar         *password;
+	gtk_dialog_response (dialog, GTK_RESPONSE_OK);
+}
 
-	priv = GET_PRIV (app);
+static void
+app_account_password_dialog_response_cb (GtkWidget *dialog,
+					 gint       response,
+					 gpointer   user_data)
+{
+	if (response == GTK_RESPONSE_OK) {
+		GossipSession *session;
+		GossipAccount *account;
+		GtkWidget     *checkbutton;
+		GtkWidget     *entry;
+		const gchar   *password;
 
-	password = gossip_password_dialog_run (account,
-					       GTK_WINDOW (priv->window));
+		session = gossip_app_get_session ();
+		account = g_object_get_data (G_OBJECT (dialog), "account");
 
-	return password;
+		checkbutton = g_object_get_data (G_OBJECT (dialog), "checkbutton");
+		entry = g_object_get_data (G_OBJECT (dialog), "entry");
+
+		password = gtk_entry_get_text (GTK_ENTRY (entry));
+
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton))) {
+			GossipAccountManager *manager;
+
+			manager = gossip_session_get_account_manager (session);
+
+			gossip_account_set_password (account, password);
+			gossip_account_manager_store (manager);
+		} else {
+			gossip_account_set_password_tmp (account, password);
+		}
+
+		/* Retry the connect */
+		gossip_session_connect (session, account, FALSE);
+	}
+
+	gtk_widget_destroy (dialog);
+}
+
+static void
+app_account_password_dialog_show (GossipAccount *account)
+{
+	GtkWidget   *dialog;
+	GtkWidget   *checkbutton;
+	GtkWidget   *entry;
+	GtkWidget   *vbox;
+	gchar       *str;
+	const gchar *name;
+
+	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+
+	str = g_strdup_printf (_("Please enter your %s account password"),
+			       gossip_account_get_name (account));
+	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (gossip_app_get_window ()),
+						     GTK_DIALOG_DESTROY_WITH_PARENT,
+						     GTK_MESSAGE_QUESTION,
+						     GTK_BUTTONS_OK_CANCEL,
+						     "<b>%s</b>",
+						     str);
+	g_free (str);
+
+	name = gossip_account_get_name (account);
+	str = g_strdup_printf (_("Logging in to account '%s'"), name);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", str);
+	g_free (str);
+
+	checkbutton = gtk_check_button_new_with_label (_("Remember Password?"));
+
+	entry = gtk_entry_new ();
+	gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);
+
+	g_signal_connect (entry, "activate",
+			  G_CALLBACK (app_account_password_dialog_activate_cb),
+			  dialog);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+
+	gtk_container_set_border_width  (GTK_CONTAINER (vbox), 6);
+
+	gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), checkbutton, FALSE, FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, FALSE, FALSE, 0);
+
+	g_object_set_data (G_OBJECT (dialog), "entry", entry);
+	g_object_set_data (G_OBJECT (dialog), "checkbutton", checkbutton);
+	g_object_set_data_full (G_OBJECT (dialog), "account", 
+				g_object_ref (account), 
+				g_object_unref);
+
+	g_signal_connect (dialog, "response", 
+			  G_CALLBACK (app_account_password_dialog_response_cb),
+			  NULL);
+
+	gtk_widget_show_all (dialog);
 }
 
 /*
@@ -1826,32 +2108,24 @@ app_popup_new_message_cb (GtkWidget *widget,
 }
 
 static void
-app_status_icon_popup_menu_cb (GtkStatusIcon  *status_icon,
+app_status_icon_popup_menu_cb (GtkStatusIcon *status_icon,
 			       guint          button,
 			       guint          activate_time,
-			       GossipApp      *app)
+			       GossipApp     *app)
 {
 	GossipAppPriv *priv;
-	GtkWidget     *submenu;
-	gboolean       show;
+	GtkWidget     *menu;
 
 	priv = GET_PRIV (app);
 
-	show = gossip_window_get_is_visible (GTK_WINDOW (priv->window));
+	menu = app_status_icon_create_menu (app);
 
-	g_signal_handlers_block_by_func (priv->popup_menu_show_list_item,
-					 app_show_hide_list_cb, app);
-	gtk_check_menu_item_set_active
-		(GTK_CHECK_MENU_ITEM (priv->popup_menu_show_list_item), show);
-	g_signal_handlers_unblock_by_func (priv->popup_menu_show_list_item,
-					   app_show_hide_list_cb, app);
+	if (!menu) {
+		/* On Mac, there is no menu (at least for now). */ 
+		return;
+	}
 
-	submenu = gossip_presence_chooser_create_menu (
-		GOSSIP_PRESENCE_CHOOSER (priv->presence_chooser));
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->popup_menu_status_item),
-				   submenu);
-
-	gtk_menu_popup (GTK_MENU (priv->popup_menu),
+	gtk_menu_popup (GTK_MENU (menu),
 			NULL, NULL,
 			gtk_status_icon_position_menu,
 			priv->status_icon,
@@ -1859,22 +2133,32 @@ app_status_icon_popup_menu_cb (GtkStatusIcon  *status_icon,
 			activate_time);
 }
 
-static void
-app_status_icon_create_menu (void)
+static GtkWidget *
+app_status_icon_create_menu (GossipApp *app)
 {
 	GossipAppPriv *priv;
 	GladeXML      *glade;
+	GtkWidget     *menu;
+	GtkWidget     *separator_item;
 	GtkWidget     *message_item;
+	GtkWidget     *show_list_item;
+	gint           connected;
+	gboolean       show;
+
+#ifdef GDK_WINDOWING_QUARTZ
+	/* Unused for now. */
+	return NULL;
+#endif
 
 	priv = GET_PRIV (app);
 
 	glade = gossip_glade_get_file ("main.glade",
 				       "tray_menu",
 				       NULL,
-				       "tray_menu", &priv->popup_menu,
-				       "tray_show_list", &priv->popup_menu_show_list_item,
+				       "tray_menu", &menu,
+				       "tray_show_list", &show_list_item,
 				       "tray_new_message", &message_item,
-				       "tray_status", &priv->popup_menu_status_item,
+				       "tray_separator1", &separator_item,
 				       NULL);
 
 	gossip_glade_connect (glade,
@@ -1883,16 +2167,33 @@ app_status_icon_create_menu (void)
 			      "tray_quit", "activate", app_chat_quit_cb,
 			      NULL);
 
-	g_signal_connect (priv->popup_menu_show_list_item, "toggled",
+	/* Status menu items */
+	gossip_session_count_accounts (priv->session,
+				       &connected,
+				       NULL,
+				       NULL);
+
+	gossip_presence_chooser_insert_menu (
+		GOSSIP_PRESENCE_CHOOSER (priv->presence_chooser),
+		menu, 
+		4, 
+		connected > 0,
+		FALSE);
+	gtk_widget_show_all (menu);
+	
+	/* New message item */
+	gtk_widget_set_sensitive (message_item, connected > 0);
+
+	/* Show/hide check menu item */
+	show = gossip_window_get_is_visible (GTK_WINDOW (priv->window));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (show_list_item), show);
+
+	g_signal_connect (show_list_item, "toggled",
 			  G_CALLBACK (app_show_hide_list_cb), app);
-
-	priv->widgets_connected = g_list_prepend (priv->widgets_connected,
-						  priv->popup_menu_status_item);
-
-	priv->widgets_connected = g_list_prepend (priv->widgets_connected,
-						  message_item);
-
+		
 	g_object_unref (glade);
+
+	return menu;
 }
 
 static void
@@ -2079,7 +2380,6 @@ gossip_app_connect (GossipAccount *account, gboolean startup)
 		/* Don't try to automatically connect if we have Network
 		 * Manager state and we are NOT connected.
 		 */
-
 		if (gossip_dbus_nm_get_state (&connected) && !connected) {
 			return;
 		}
@@ -2089,6 +2389,23 @@ gossip_app_connect (GossipAccount *account, gboolean startup)
 	gossip_session_connect (priv->session, account, startup);
 }
 
+void
+gossip_app_disconnect (GossipAccount *account)
+{
+	GossipAppPriv        *priv;
+	GossipAccountManager *manager;
+	
+	priv = GET_PRIV (app);
+
+	manager = gossip_session_get_account_manager (priv->session);
+	if (gossip_account_manager_get_count (manager) < 1) {
+		/* Show the accounts dialog instead */
+		return;
+	}
+	
+	gossip_session_disconnect (priv->session, account);
+}
+
 /* Test hack to add support for disconnecting all accounts and then connect
  * them again due to if the net goes up and down.
  */
@@ -2168,16 +2485,6 @@ gossip_app_get (void)
 	return app;
 }
 
-static void
-app_disconnect (void)
-{
-	GossipAppPriv *priv;
-
-	priv = GET_PRIV (app);
-
-	gossip_session_disconnect (priv->session, NULL);
-}
-
 static void
 app_connection_items_setup (GladeXML *glade)
 {
@@ -2185,7 +2492,8 @@ app_connection_items_setup (GladeXML *glade)
 
 	const gchar   *widgets_connected[] = {
 		"chat_disconnect",
-		"room",
+		"room_join_new", 
+		"room_join_favorites", 
 		"chat_new_message",
 		"chat_add_contact",
 		"edit_personal_information"
@@ -2216,6 +2524,148 @@ app_connection_items_setup (GladeXML *glade)
 	priv->widgets_disconnected = list;
 }
 
+static void
+app_connection_menu_update (void)
+{
+	GossipAppPriv        *priv;
+	GossipAccountManager *manager;
+	GList                *accounts;
+	GList                *l;
+	GtkWidget            *menu_connect = NULL;
+	GtkWidget            *menu_disconnect = NULL;
+	gboolean              connected;
+
+	priv = GET_PRIV (app);
+		
+	manager = gossip_session_get_account_manager (priv->session);
+	accounts = gossip_account_manager_get_accounts (manager);
+
+	/* We always disconnect these in case we move from 1 account
+	 * to many accounts, if we don't do this, we start connecting
+	 * when we are showing submenus.
+	 */
+	g_signal_handlers_disconnect_by_func (priv->chat_connect, 
+					      app_chat_connect_activate_cb,
+					      NULL);
+	g_signal_handlers_disconnect_by_func (priv->chat_disconnect, 
+					      app_chat_disconnect_activate_cb,
+					      NULL);
+
+	/* If we have one account we do things quite differently. We
+	 * only have one menu item and toggle it.
+	 */
+	if (g_list_length (accounts) == 0) {
+		gtk_widget_show (priv->chat_connect);
+		gtk_widget_hide (priv->chat_disconnect);
+
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_connect), NULL);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_disconnect), NULL);
+
+		g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
+		g_list_free (accounts);
+
+		return;
+	}
+
+	if (g_list_length (accounts) == 1) {
+		connected = gossip_session_is_connected (priv->session, accounts->data);
+
+		if (connected) {
+			gtk_widget_hide (priv->chat_connect);
+			gtk_widget_show (priv->chat_disconnect);
+		} else {
+			gtk_widget_show (priv->chat_connect);
+			gtk_widget_hide (priv->chat_disconnect);
+		}			
+
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_connect), NULL);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_disconnect), NULL);
+
+		g_object_set_data_full (G_OBJECT (priv->chat_connect), "account", 
+					g_object_ref (accounts->data), 
+					g_object_unref);
+		g_object_set_data_full (G_OBJECT (priv->chat_disconnect), "account", 
+					g_object_ref (accounts->data), 
+					g_object_unref);
+
+		g_signal_connect (priv->chat_connect, "activate",
+				  G_CALLBACK (app_chat_connect_activate_cb),
+				  NULL);
+		g_signal_connect (priv->chat_disconnect, "activate",
+				  G_CALLBACK (app_chat_disconnect_activate_cb),
+				  NULL);
+
+		g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
+		g_list_free (accounts);
+
+		return;
+	}
+
+	/* If we have multiple accounts we show a connect-> and a
+	 * disconnect-> menu instead. 
+	 */
+	gtk_widget_show (priv->chat_connect);
+	gtk_widget_show (priv->chat_disconnect);
+
+	for (l = accounts; l; l = l->next) {
+		GtkWidget   *item;
+		GtkWidget   *image;
+		GCallback    callback;
+		const gchar *name;
+		GdkPixbuf   *pixbuf;
+
+		name = gossip_account_get_name (l->data);
+		item = gtk_image_menu_item_new_with_label (name);
+
+		connected = gossip_session_is_connected (priv->session, l->data);
+		pixbuf = gossip_account_status_create_pixbuf (l->data, 
+							      GTK_ICON_SIZE_MENU,
+							      connected);
+		image = gtk_image_new_from_pixbuf (pixbuf);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		g_object_unref (pixbuf);
+
+		gtk_widget_show (item);
+
+		if (gossip_session_is_connected (priv->session, l->data) || 
+		    gossip_session_is_connecting (priv->session, l->data)) {
+			/* Create disconnect account menu item */
+			if (!menu_disconnect) {
+				menu_disconnect = gtk_menu_new ();
+				gtk_widget_show (menu_disconnect);
+			}
+
+			callback = G_CALLBACK (app_chat_disconnect_activate_cb);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu_disconnect), item);
+		} else {
+			/* Create connect account menu item */
+			if (!menu_connect) {
+				menu_connect = gtk_menu_new ();
+				gtk_widget_show (menu_connect);
+			}
+			
+			callback = G_CALLBACK (app_chat_connect_activate_cb);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu_connect), item);
+		}
+
+		g_object_set_data_full (G_OBJECT (item), "account", 
+					g_object_ref (l->data), 
+					g_object_unref);
+		g_signal_connect (item, "activate",
+				  callback,
+				  NULL);
+	}
+
+	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
+	g_list_free (accounts);
+
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_connect), menu_connect);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_disconnect), menu_disconnect);
+
+	gtk_widget_set_sensitive (priv->chat_connect, menu_connect != NULL);
+	gtk_widget_set_sensitive (priv->chat_disconnect, menu_disconnect != NULL);
+}
+
 static void
 app_connection_items_update (void)
 {
@@ -2226,6 +2676,9 @@ app_connection_items_update (void)
 
 	priv = GET_PRIV (app);
 
+	/* Set up connect/disconnect menus */
+	app_connection_menu_update ();
+
 	/* Get account count for:
 	 *  - connected and disabled,
 	 *  - connected and enabled
@@ -2556,6 +3009,14 @@ app_event_added_cb (GossipEventManager *manager,
 	gossip_request_user_attention ();
 }
 
+static void
+app_accounts_updated_cb (GossipAccountManager  *manager,
+			 GossipAccount         *account,
+			 gpointer               user_data)
+{
+	app_connection_items_update ();
+}
+
 static void
 app_contact_activated_cb (GossipContactList *contact_list,
 			  GossipContact     *contact,
