@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2003 Imendio AB
- * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,6 +16,11 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
+ *          Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  */
 
 #ifndef __GOSSIP_CHAT_H__
@@ -24,7 +28,9 @@
 
 #include <glib-object.h>
 
+#include <libgossip/gossip-chatroom.h>
 #include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-message.h>
 
 G_BEGIN_DECLS
 
@@ -35,13 +41,13 @@ G_BEGIN_DECLS
 #define GOSSIP_IS_CHAT_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GOSSIP_TYPE_CHAT))
 #define GOSSIP_CHAT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GOSSIP_TYPE_CHAT, GossipChatClass))
 
-typedef struct _GossipChat GossipChat;
-typedef struct _GossipChatClass GossipChatClass;
-typedef struct _GossipChatPriv GossipChatPriv;
+typedef struct _GossipChat       GossipChat;
+typedef struct _GossipChatClass  GossipChatClass;
+typedef struct _GossipChatPriv   GossipChatPriv;
 
-#include "gossip-chat-view.h"
-#include "gossip-chat-window.h"
-#include "gossip-spell.h"
+#include "gossip-chat-view.h" 
+#include "gossip-chat-window.h" 
+#include "gossip-spell.h" 
 
 struct _GossipChat {
 	GObject         parent;
@@ -139,6 +145,12 @@ gboolean          gossip_chat_should_play_sound     (GossipChat       *chat);
 gboolean          gossip_chat_should_highlight_nick (GossipMessage    *message,
 						     GossipContact    *my_contact);
 
+/* Sent messages API, for buffer recall */
+void              gossip_chat_sent_message_add      (GossipChat       *chat,
+						     const gchar      *str);
+const gchar *     gossip_chat_sent_message_get_next (GossipChat       *chat);
+const gchar *     gossip_chat_sent_message_get_last (GossipChat       *chat);
+
 G_END_DECLS
 
 #endif /* __GOSSIP_CHAT_H__ */
