@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Martyn Russell <ginxd@btopenworld.com>
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,13 +16,19 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_VCARD_DIALOG_H__
 #define __GOSSIP_VCARD_DIALOG_H__
 
-#include "gossip-app.h"
+G_BEGIN_DECLS
 
 void gossip_vcard_dialog_show (GtkWindow *parent);
 
+G_END_DECLS
+
 #endif /* __GOSSIP_VCARD_DIALOG_H__ */
